/* リスト3.3 MQO形式の3Dモデルを描画するサンプルプログラム */

import javax.media.opengl.*;  // JavaのOpenGL（JOGL)の機能を使うために必要
import processing.opengl.*;   // ProcessingのOpenGLの機能を使うために必要
import jp.nyatla.kGLModel.*;                 // MQOローダを使うために必要
import jp.nyatla.kGLModel.contentprovider.*; // MQOローダを使うために必要

KGLModelData     model_data;        // モデルデータ
ContentProvider  content_provider;  // ファイル読み込み

void setup() {
  size(400, 300, OPENGL);
  content_provider = new LocalContentProvider(this,"C:\\ninja.mqo");
  PGraphicsOpenGL pgl = (PGraphicsOpenGL) g;
  GL gl = pgl.beginGL();
  model_data = KGLModelData.createGLModelPs( 
                this, gl, null, 
                this.content_provider,1.0,
                KGLExtensionCheck.IsExtensionSupported( gl,
                "GL_ARB_vertex_buffer_object"),
                true );
  pgl.endGL();
}

void draw() {
  background(128);
  
  // 視点の設定
  camera(mouseX, mouseY, 200, width/2.0, height/2.0, 0, 0, 1, 0);  

  PGraphicsOpenGL pgl = (PGraphicsOpenGL) g;
  GL gl = pgl.beginGL();
  
    // テクスチャとポリゴンの描画に関する設定
    gl.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_MODULATE);
    gl.glEnable(GL.GL_CULL_FACE);
    gl.glCullFace(GL.GL_FRONT);
  
    // 光源の設定
    mySetLight(gl, 0, 0, 1000);
    
    // 3Dモデルの表示位置と向き
    gl.glTranslatef( width/2, 0.8*height, 0); // 画面中央に立たせるために移動
    gl.glRotatef(180, 1, 0, 0);                // 立たせるために回転
  
    // 3Dモデルの描画
    model_data.draw() ;
    
  pgl.endGL();
}

// 光源の設定を行う関数
void mySetLight(GL gl, float x, float y, float z) {
  float[] light_diffuse  = { 0.2, 0.2, 0.2, 1.0 };  // 拡散反射光
  float[] light_specular = { 0.3, 0.3, 0.3, 1.0 };  // 鏡面反射光
  float[] light_ambient  = { 0.3, 0.3, 0.3, 0.1 };  // 環境光
  float[] light_position = { x, y, z, 0.0 };        // 位置と種類

  gl.glLightfv( gl.GL_LIGHT0, gl.GL_DIFFUSE,  light_diffuse, 0); // 拡散反射光
  gl.glLightfv( gl.GL_LIGHT0, gl.GL_SPECULAR, light_specular,0); // 鏡面反射光
  gl.glLightfv( gl.GL_LIGHT0, gl.GL_AMBIENT,  light_ambient, 0); // 環境光
  gl.glLightfv( gl.GL_LIGHT0, gl.GL_POSITION, light_position,0); // 位置と種類
  gl.glShadeModel( gl.GL_SMOOTH ); // シェーディングの種類の設定
  gl.glEnable( gl.GL_LIGHT0 );     // 光源0の有効化
  gl.glEnable( gl.GL_LIGHTING );   // ライティングの有効化 
}
