/* リスト3.19 姿勢の認識と身体部位の取得 */

import SimpleOpenNI.*;

SimpleOpenNI  context;
color[] userColors = { color(0,0,0), color(255,0,0), color(0,255,0),
color(0,0,255) };

void setup() {
  context = new SimpleOpenNI(this);                   // SimpleOpenNIの初期化
  context.setMirror(true);                            // 画像のミラー反転有効化
  context.enableDepth();                              // 距離画像有効化
  context.enableRGB();                                // カラー画像有効化
  context.enableUser(SimpleOpenNI.SKEL_PROFILE_ALL);  // ユーザトラッキング有効化
  context.alternativeViewPointDepthToImage();         // 視点を一致させる
  size(context.rgbWidth(), context.rgbHeight());      // 画面サイズの設定
  strokeWeight(3);                                    // 線の太さの設定
}

void draw() {
  context.update();              // カメラの更新
  image(context.rgbImage(),0,0); // カラー画像の表示
  
  // トラッキングできていたらスケルトンを表示
  for (int userId=1; userId<=context.getNumberOfUsers(); userId++) {
    if( context.isTrackingSkeleton(userId) ) {
      color user_color =userColors[userId % userColors.length];
      stroke(user_color);             // 線の色
      fill(user_color);               // 塗りつぶしの色
      drawSkeleton(userId);           // スケルトン描画
    }
  }
}

// スケルトンの描画
void drawSkeleton(int userId) { 
  // 頭-首
  context.drawLimb(userId, SimpleOpenNI.SKEL_HEAD, SimpleOpenNI.SKEL_NECK);
  
  // 首-左肩，左肩-左肘，左肘-左手
  context.drawLimb(userId, SimpleOpenNI.SKEL_NECK, SimpleOpenNI.SKEL_LEFT_SHOULDER);
  context.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_SHOULDER, SimpleOpenNI.SKEL_LEFT_ELBOW);
  context.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_ELBOW, SimpleOpenNI.SKEL_LEFT_HAND);

  // 首-右肩，右肩-右肘，右肘-右手
  context.drawLimb(userId, SimpleOpenNI.SKEL_NECK, SimpleOpenNI.SKEL_RIGHT_SHOULDER);
  context.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_SHOULDER, SimpleOpenNI.SKEL_RIGHT_ELBOW);
  context.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_ELBOW, SimpleOpenNI.SKEL_RIGHT_HAND);

  // 左肩-胴，右肩-胴
  context.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_SHOULDER, SimpleOpenNI.SKEL_TORSO);
  context.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_SHOULDER, SimpleOpenNI.SKEL_TORSO);

  // 胴-左腰，左腰-左膝，左膝-左脚
  context.drawLimb(userId, SimpleOpenNI.SKEL_TORSO, SimpleOpenNI.SKEL_LEFT_HIP);
  context.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_HIP, SimpleOpenNI.SKEL_LEFT_KNEE);
  context.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_KNEE, SimpleOpenNI.SKEL_LEFT_FOOT);

  // 胴-右腰，右腰-右膝，右膝-右脚
  context.drawLimb(userId, SimpleOpenNI.SKEL_TORSO, SimpleOpenNI.SKEL_RIGHT_HIP);
  context.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_HIP, SimpleOpenNI.SKEL_RIGHT_KNEE);
  context.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_KNEE, SimpleOpenNI.SKEL_RIGHT_FOOT);  

  // 右手の３次元位置を取得
  PVector realPos = new PVector();
  context.getJointPositionSkeleton(userId,SimpleOpenNI.SKEL_RIGHT_HAND,realPos);
  
  // 実世界の座標値から，画面座標系の座標値に変換し，円を描画
  PVector screenPos = new PVector();  
  context.convertRealWorldToProjective(realPos,screenPos);
  ellipse( screenPos.x, screenPos.y, 20, 20);
}

// ----------------------- 以下，SimpleOpenNIのイベント処理 -----------------------

// 新しくユーザが見つかった際の処理
void onNewUser(int userId) {
  context.startPoseDetection("Psi",userId); // Ψポーズの検出開始
}

// ポーズ開始時の処理
void onStartPose(String pose,int userId) {
  context.stopPoseDetection(userId);                // ポーズ検出を停止
  context.requestCalibrationSkeleton(userId, true); // キャリブレーションを指示
}

// キャリブレーションが終了した際の処理
void onEndCalibration(int userId, boolean successfull) {
  if (successfull) { 
    context.startTrackingSkeleton(userId);  // 成功ならトラッキング開始
  } else {
    context.startPoseDetection("Psi",userId); // 失敗ならΨポーズの検出開始
  }
}
