/* リスト4.2 AR距離計のプログラム */

import jp.nyatla.nyar4psg.*;  // NyAR4psg
import processing.video.*;    // ビデオライブラリ
import processing.serial.*;   // シリアル通信ライブラリ
import cc.arduino.*;          // Arduinoライブラリ

Arduino arduino; // Arduino
Capture cam;     // キャプチャ
MultiMarker ar;  // ARマーカに関する処理をするオブジェクト
int id;          // マーカに割り当てられるID番号

// センサが接続されているアナログピンの番号
int sensorPin = 0;

// マーカに対するセンサの位置（単位：mm）
float offset_x = 45.0;
float offset_y = 0.0;
float offset_z = -30.0;

// キャリブレーションで求めた定数
float a = 118200;
float b = -11.5;

void setup() {
  // 画面の設定
  size(640,480,P3D); 

  // ARをやるための準備
  cam = new Capture(this, width, height);
  ar = new MultiMarker(this, width, height, "camera_para.dat", NyAR4PsgConfig.CONFIG_PSG);
  
  // マーカの設定（使用するマーカに合わせて値を変更してください）
  id = ar.addARMarker("AR_measure.patt", 50); 

  // Arduinoの設定（自分の環境に合わせてポート番号を変更してください）
  arduino = new Arduino(this, Arduino.list()[0], 57600);
  
  // フォントの設定（任意のフォントを設定してください）
  PFont font = loadFont("HGSoeiKakugothicUB-50.vlw");
  textFont(font);
}

void draw() {
  // カメラの用意ができていないときは処理をせずに出る
  if (cam.available()==false) return;

  cam.read();             // カメラ画像の読み込み
  background(0);          // 画面の初期化
  ar.drawBackground(cam); // 背景画像の描画
  ar.detect(cam);         // マーカ認識
  
  // センサ入力からの距離の計算
  float value = arduino.analogRead(sensorPin);  
  float distance = a / (value + b);

  // マーカが認識されていたら
  if ( ar.isExistMarker(id) && distance > 0 ) {
    ar.beginTransform(id);
 
     // ビームの描画  
      strokeWeight(15);
      stroke(255,0,0,128);   
      line( offset_x, offset_y, offset_z,
            distance + offset_x, offset_y, offset_z );
  
      // 距離の表示
      rotateZ(PI);
      rotateX(-PI/2);
      fill(255,0,0,127);
      textSize(40);
      translate(-100,-50);
      text("距離:"+int(distance)+"mm",0,0,0);

    ar.endTransform();
  }
}
