/* リスト4.4 ARリモコンのプログラム */

import jp.nyatla.nyar4psg.*; // ARライブラリ
import processing.video.*;   // ビデオライブラリ
import processing.serial.*;  // シリアル通信ライブラリ
import cc.arduino.*;         // Arduinoライブラリ
import picking.*;            // ピッキングライブラリ

Capture      cam;           // キャプチャ
MultiMarker  ar;            // AR処理
Picker       picker;        // ピッキング
Arduino      arduino;       // Arduino
int          selected = -1; // 選択されているオブジェクトの番号

// オブジェクトに関する情報
int[]      port     = { 2, 3, 4 }; // Arduinoの出力ポート
boolean[]  state    = { false, false, false }; // 家電のON・OFFの状態
String[]   name     = { "扇風機", "フロアランプ", "行燈" };
float[][]  box_size = { {250,250,100}, {400,1500,400}, {250,500,100} };
float[][]  offset   = { {0, 0,0}, {0,-250,0}, {0,-250,0} };
color[]    colors   = { color(255,0,0,60), color(0,255,0,60), 
                        color(255,255,0,60) };

void setup(){
  // 画面サイズとキャプチャの設定
  size(800, 600, P3D);
  cam = new CaptureDS(this, width, height);

  // ARマーカの設定
  ar = new MultiMarker(this, width, height, "camera_para.dat", NyAR4PsgConfig.CONFIG_PSG );
  ar.addARMarker("kaze", 80);
  ar.addARMarker("akari", 80);
  ar.addARMarker("tomoshibi", 80);

  // Arduinoの設定 （ポート番号は自分の環境に合わせてください）
  arduino = new Arduino(this, Arduino.list()[0], 57600);
  for (int i=0; i<port.length; i++) {
    arduino.pinMode( port[i], Arduino.OUTPUT );
  }

  // フォントの設定（任意のフォントを設定してください）
  PFont font = loadFont("HGSoeiKakugothicUB-50.vlw");
  textFont(font);
  textSize(25);

  // ピッキングの準備
  picker = new Picker(this);
}

void draw(){
  if (cam.available()==false) return;
  
  cam.read();             // カメラ画像の読み込み
  background(0);          // 画面の初期化
  ar.drawBackground(cam); // 背景画像の描画
  ar.detect(cam);         // マーカ認識

  for (int i=0; i<3; i++) {
    if ( ar.isExistMarker(i) ) {
      picker.start(i); // ピッキング用のIDを付与する
      ar.beginTransform(i);
        if ( i == selected ) {
          fill( colors[i] );
        } else {
          fill(0,0,0,0);
        }
        translate( offset[i][0], offset[i][1], offset[i][2] );
        box( box_size[i][0], box_size[i][1],box_size[i][2] );
      ar.endTransform();
      picker.stop();
    }
  }

  // 家電の名前を表示
  for (int i=0; i<3; i++) {
    if ( i == selected ) {
      fill(0,0,0);
      text(name[i],mouseX,mouseY-10);
    }
  }

  // ピッキング判定
  selected = picker.get(mouseX, mouseY);
}

// 画面をクリックしたら選択されている装置をOn/Off
void mouseClicked() {
  if ( selected >= 0 ) {
    if ( state[selected] == false ) {
      arduino.digitalWrite( port[selected], Arduino.HIGH );
    } else {
      arduino.digitalWrite( port[selected], Arduino.LOW );
    }
    state[selected] = !state[selected];
  }
}
