/* リスト4.7 HackAR クライアントのプログラム（操作する側） */

import processing.net.*;     // ネットワークライブラリ
import processing.video.*;   // ビデオライブラリ
import jp.nyatla.nyar4psg.*; // ARライブラリ

MultiMarker ar;     // ARマーカ
Capture     cam;    // カメラ
Client      client; // クライアント

// 以下は自分の環境に合わせて設定してください
int   NyID = 21;                     // NyIDマーカのID番号
int   marker_size = 120;             // マーカの物理サイズ（単位：mm）
float display_actual_width = 1150.0; // ディスプレイの物理サイズ（横）（単位：mm）
float display_actual_height = 650.0; // ディスプレイの物理サイズ（縦）（単位：mm）
String ipaddress = "192.168.12.345"; // サーバを起動しているPCのIPアドレス

void setup() {
  // 画面サイズの設定
  size(640, 480, P3D);
  
  // AR処理の準備
  cam = new Capture(this, width, height); 
  ar = new MultiMarker(this, width, height, "camera_para.dat", NyAR4PsgConfig.CONFIG_PSG);
  ar.addNyIdMarker(NyID, marker_size);
  ar.setThreshold(80); // マーカ検出に用いる閾値の設定

  // クライアントの準備
  client = new Client(this, ipaddress, 12345);
}

void draw() {  
  if (cam.available()==false) return; // カメラの用意ができていないときは処理をしない
  cam.read();                         // カメラ画像の読み込み
  background(0);                      // 画面の初期化
  ar.drawBackground(cam);             // 背景画像の描画
  ar.detect(cam);                     // マーカ認識

  // マーカを認識したら
  if ( ar.isExistMarker(0) ) {
    
    // ディスプレイを囲う四角形を描画
    ar.beginTransform(0);
      fill(255,255,255,50);
      strokeWeight(4);
      stroke(255,0,0);
      rect(0, 0, display_actual_width, display_actual_height);
    ar.endTransform();

    // 画面上のマウスカーソルが，実世界中でどこにあるのかを計算する
    PVector pos3d = ar.screen2MarkerCoordSystem( 0, mouseX, mouseY );

    // 正規化した座標値を計算
    float nx = pos3d.x/display_actual_width;
    float ny = pos3d.y/display_actual_height;
    
    // 座標値とボタンの押下状態をサーバに送信
    if ( nx>=0 && nx<1.0 && ny>=0 && ny<1.0 ) {
      client.write( nx  + " " + ny + " " + mousePressed +  "\n");    
    }
  }
}
